%function [] = quick_test_IRenyi()
%Quick test for Renyi mutual information estimators: analytical expression vs estimated value as a function of the sample number. In the test, normal variables are considered.

%Copyright (C) 2013 Zoltan Szabo ("http://www.gatsby.ucl.ac.uk/~szabo/", "zoltan (dot) szabo (at) gatsby (dot) ucl (dot) ac (dot) uk")
%
%This file is part of the ITE (Information Theoretical Estimators) Matlab/Octave toolbox.
%
%ITE is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by
%the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
%
%This software is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
%MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
%
%You should have received a copy of the GNU General Public License along with ITE. If not, see <http://www.gnu.org/licenses/>.

%clear start:
    clear all; close all;
    
%parameters:
    distr = 'normal'; %fixed
    alpha = 0.7; %parameter of Renyi mutual information, \ne 1
    d = 2; %>=2; dimension of the distribution
    num_of_samples_v = [100:500:10*1000]; %sample numbers used for estimation
    %estimator (of Renyi mutual information), meta:
        cost_name = 'Renyi_HRenyi';
        %cost_name = 'Renyi_DRenyi';
    
%initialization:
    num_of_samples_max = num_of_samples_v(end);
    L = length(num_of_samples_v);
    co = I_initialization(cost_name,1,{'alpha',alpha}); %{'alpha',alpha_H}: set the 'alpha' field
    I_hat_v = zeros(L,1); %vector of estimated Renyi mutual information
    
%distr, d -> samples (Y), analytical formula for the Renyi mutual information (I):
    switch distr
        case 'normal'
            %expectation:
                e = rand(d,1);
                %e = zeros(d,1);
            %(random) linear transformation applied to the data:                
                A = rand(d); 
                %A = eye(d);  %do not transform the data
            %covariance matrix:
                cov_mtx = A * A.';
            %generate samples:
                Y = A * randn(d,num_of_samples_max) + repmat(e,1,num_of_samples_max); %AxN(0,I)+e
            %analytical value of Renyi mutual information:
                t1 = -alpha / 2 * log(det(cov_mtx));
                t2 = -(1-alpha) / 2 * log(prod(diag(cov_mtx)));
                t3 = log( det( alpha*inv(cov_mtx) + (1-alpha)*diag(1./diag(cov_mtx)) ) ) / 2;
                I = 1 / (alpha-1) * (t1+t2-t3);            
        otherwise
            error('Distribution=?');            
    end
    
%estimation:
    Tk = 0;%index of the sample number examined
    for num_of_samples = num_of_samples_v
        Tk = Tk + 1;
        I_hat_v(Tk) = I_estimation(Y(:,1:num_of_samples),ones(d,1),co);
        disp(strcat('Tk=',num2str(Tk),'/',num2str(L)));
    end
    
%plot:
    plot(num_of_samples_v,I_hat_v,'r',num_of_samples_v,I*ones(L,1),'g');
    legend({'estimation','analytical value'});
    xlabel('Number of samples');
    ylabel('Renyi mutual information');
 
